"""

    Unit test cases for DataGridField column definition manipulation

    Copyright 2006 Mikko Ohtamaa

"""
import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.DataGridField.tests.DataGridTestCase import DataGridTestCase
from Products.Archetypes.public import *

class TestColumns(DataGridTestCase):
    """ Unit test cases for DataGridField column definition manipulation """

    def afterSetUp(self):
        self.folder.invokeFactory('DataGridDemoType', 'demo')
        self.demo = self.folder.demo

        self.folder.invokeFactory('DataGridDemoType2', 'demo2')
        self.demo2 = self.folder.demo2

        self.folder.invokeFactory('InvalidDataGridDemoType', 'invalid_demo')
        self.invalid_demo = self.folder.invalid_demo


    def testFill(self):
        """ Test that it is possible to enter data """
        self.demo.setDemoField([{'column1':'a', 'column2':'b', 'The third':'c'},
                                {'column1':'d', 'column2':'e', 'The third':'f'}])
        self.field = self.demo.getField('DemoField')


    def testGetVocabulary(self):
        """ Test if vocabulary is received correctly for a select column
        """

        field = self.demo2.getField("DemoField2")
        col = field.widget.getColumnDefinition(field, "select_sample")
        vocab = col.getVocabulary(self.demo2)

        self.assertEqual(vocab.keys()[0], 'sample')
        self.assertEqual(vocab.keys()[1], 'sample2')

        self.assertEqual(vocab.values()[0], 'Sample value 1')
        self.assertEqual(vocab.values()[1], 'Sample value 2')

    def testInvalidWidgetColumnDefinition(self):
        """ Try to get column definitions when there is field<->widget mismatch"""

        try:
            self.invalid_demo.getColumnLabels(self.invalid_demo, self)
            passed = False
        except AttributeError:
            # widget is missing column select_sample, should raise an error
            passed = True
            pass

        self.failUnless(passed, "Missing widget column ids were not catched")


    def testGetColumnDefinition(self):
        """ Just get column definitions"""

        field = self.demo2.getField("DemoField2")
        col = field.widget.getColumnDefinition(field, "select_sample")

        # backward compatibility: no explict column defnition given
        # so it should have been constructed automatically
        field = self.demo2.getField("DemoField2")
        col = field.widget.getColumnDefinition(field, "column1")

    def testGetColumnNames(self):
        """ Get user friendly column names """        
        field = self.demo2.getField("DemoField2")
        names = field.widget.getColumnLabels(field, self)
        self.assertEqual(names, ["Toholampi city rox","My friendly name","Friendly name"])

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestColumns))
    return suite

if __name__ == '__main__':
    framework()



"""
    DataGridField field class
    
    Copyright 2006 DataGridField authors, see documentation for details

"""
from __future__ import nested_scopes

__docformat__ = 'epytext'
__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'

import StringIO
import logging
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Field import encode
from Products.Archetypes.Field import registerField
from Products.DataGridField import DataGridWidget
from Products.DataGridField.interfaces import IDataGridField

# Our logger object
logger = logging.getLogger('DataGridField')
logger.debug("DataGridField loading")

class DataGridField(ObjectField):
    """ Table field with undefined number of rows

    DataGridField provides an user fillable table with fixed
    columns and undefined number of rows.

    Data is maintained internally:
        - DataGridField.value is a list
        - Each list item represents a row
        - Each row is a dictionary using column names as a key
    """
    __implements__ = (ObjectField.__implements__, IDataGridField,)

    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'datagrid',
        'mode' : 'rw',
        # inital data of the field in form sequence of dicts
        'default' : ({},),
        
        'widget' : DataGridWidget,
        
        # Column id list (sequence of strings)
        'columns' : ('default',),

        # sequence of FixedRow instances.
        # See FixedRow class documentation   
        'fixed_rows' : [],
    
        # User can append new rows. Currently UI feature. This is not yet checked 
        # at application level in set().        
        'allow_insert' : True,
        
        # User can delete rows. This is currently UI feature (red delete button).
        # This is not yet checked at application level in set().
        'allow_delete' : True,
        
        # User can reorder rows. This is currently UI feature (ordering buttons).
        # This is not yet checked at application level in set().      
        'allow_reorder' : True,
         
         # If true all the contents of the DataGridField is concatenated
         # to searchable text and given to text indexer
        'searchable' : False,
        
        # Set to false to allow empty rows in the data.
        # Needed for auto insert feature
        'allow_empty_rows' : True,
        
        # Set to true to hifhligh odd/even rows in edit/view form
        'allow_oddeven' : False,
        })

    security = ClassSecurityInfo()


    def __init__(self, name=None, **kwargs):
        """ Create DataGridField instance
        """

        # call super constructor
        ObjectField.__init__(self, name, **kwargs)

    def getColumnIds(self):
        """ Return list of column ids """
        return self.columns
    
    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        """
        The passed in object should be a records object, or a sequence of dictionaries
        """
        
        # Help to localize problems in Zope trace back
        __traceback_info__ = value, type(value)

        # we sanitize the values
        cleaned = []
        doSort = False
        
        logging.debug("Setting DGF value to " + str(value))
        
        if value == ({},):
            # With some Plone versions, it looks like that AT init
            # causes DGF to get one empty dictionary as the base value
            # and later, it will be appended as a cleaned row below if 
            # we don't filter out it here. 
            value = []
            
        
        if isinstance(value, basestring):
            # In the field mutator (set) the
            # passed value is not always a record, but sometimes a string.
            # In fact the RFC822Marshaller passes a string. 
            
            logging.debug("Doing string marshalling")
            
            records = []
            dict = {}
            rows = value.strip("{}").split(',')
            for atuple in rows:
                key, val = atuple.split(":", 1)
                key, val = key.strip(), val.strip()
                try:
                    dict[key].append(val)
                except AttributeError:
                    dict[key] = [dict[key], val]
                except KeyError:
                    dict[key] = val

            if len(dict) > 0:
                records.append(dict)

            value = records         
        else:
            
            # Passed in value is a HTML form data
            # from DataGridWidget. Value is Python array,
            # each item being a dictionary with column_name : value mappins
            # + orderinder which is used in JS reordering
            
            for row in value:
                order = row.get('orderindex_', None)
                
                empty = True
                                
                if order != "template_row_marker":
                    # don't process hidden template row as
                    # input data                     
                    
                    val = {}
                    for col in self.getColumnIds():
                        val[col] = (row.get(col,'')).strip()
                        
                        if val[col] != '':
                            empty = False
                                                                        
                    if order is not None:                        
                        try:
                            order = int(order)
                            doSort = True
                        except ValueError:
                            pass

                    # create sortable tuples
                    if (not self.allow_empty_rows) and empty:
                        logger.debug("Filtered out an empty row")
                    else:
                        logger.debug("Appending cleaned row:" + str(val))
                        cleaned.append((order, val.copy()))

            if doSort:
                cleaned.sort()

            # remove order keys when sorting is complete
            value = tuple([x for (throwaway, x) in cleaned])

        # fill in data
        ObjectField.set(self, instance, value, **kwargs)

    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        """ Return DataGridField value

        Value is a list object of rows.

        If parameter mimetype == 'text/plain' is passed,
        a string containing all cell values concatenated together is returned.
        This is for site indexing services (DataGridField.searchable = true).
        """

        if(kwargs.has_key('mimetype') and kwargs['mimetype'] == 'text/plain'):
            # Data is returned for text indexing
            # Concatenate all cell values
            buffer = StringIO.StringIO()

            value = ObjectField.get(self, instance, **kwargs) or ()
            value = self.resetFixedRows(instance, value)
            
            for row in value:
                for col in self.getColumnIds():
                    buffer.write(row.get(col,''))
                    # separate the last word of a cell
                    # and the first of the next cell
                    buffer.write(' ')

            return encode(buffer.getvalue(), instance, **kwargs)

        else:
            # Return list containing all encoded rows
            value = ObjectField.get(self, instance, **kwargs) or ()
            value = self.resetFixedRows(instance, value)
            
            data = [encode(v, instance, **kwargs) for v in value]
            
            return tuple(data)

    security.declarePrivate('getRaw')
    def getRaw(self, instance, **kwargs):
        return self.get(instance, **kwargs)

    security.declarePublic('get_size')
    def get_size(self, instance):
        """Get size of the stored data used for get_size in BaseObject
        """
        size=0
        for line in self.get(instance):
            size+=len(str(line))
        return size

    # Grid API as defined in interfaces.py

    security.declarePublic('getColumns')
    def getColumns(self, instance):
        return self.columns

    security.declarePublic('getRowCount')
    def getRowCount(self, instance):
        return len(self.get(instance))

    def getRow(self, instance, rowNumber):
        """Get the given row by number. The first row is rowNumber 0. Returns
        a dict of the row.
        """
        data = self.get(instance)
        if rowNumber > len(data):
            raise IndexError, "Tried to access row %d when there are %d available" % (rowNumber, len(data))
        return data[rowNumber]

    def getColumn(self, instance, columnName):
        """Get data in the given column by name as a tuple of values.
        """
        data = self.get(instance)
        col = []
        for row in data:
            col.append(row[columnName])
        return tuple(col)

    def search(self, instance, key=None, **kwargs):
        """Search for rows. If key is given look for this value in the first
        column. If key is not given, at least one kwarg must be given,
        specifying column names and values. For example,
            search(surname='Jones', title='Mr')
        will return all rows with the surname and title matching Mr. Jones.
        Always returns a tuple of dicts.
        """
        data = self.get(instance)
        if key is not None:
            kwargs[self.getColumnIds()[0]] = key
        matches = []
        for r in data:
            match = True
            for k, v in kwargs.items():
                if r[k] != v:
                    match = False
            if match:
                matches.append(r)
        return tuple(matches)


    def lookup(self, instance, key, column, lookupColumn=None):
        """Look for the given key in the column specified by lookupColumn.
        If no lookupColumn is given, look in the first column. For the first row
        found, return the value stored in the corresponding column as given by
        the 'column' parameter. Returns None if the key could not be found.

        (if you've used Excel, this is similar to VLOOKUP)
        """
        data = self.get(instance)
        if lookupColumn is None:
            lookupColumn = self.columns[0]
        for r in data:
            if r[lookupColumn] == key:
                return r[column]
        return None


    def getAsDisplayList(self, instance, keyCol=None, valueCol=None):
        """Get two columns of each row as a DisplayList - the key columns is
        keyCol, and the value column is valueCol. If these are not given,
        use the first two columns, respectively.
        """
        data = self.get(instance)

        if keyCol is None:
            keyCol = self.getColumnIds()[0]
        if valueCol is None:
            valueCol = self.getColumnIds()[1]

        lst = DisplayList()
        for r in data:
            lst.add(r[keyCol], r[valueCol])

        return lst

    def getAsGrid(self, instance):
        """Return a tuple of tuples - the outer tuple has one element
        per row in the grid, the inner tuple has one element per column
        in that row.
        """
        data = self.get(instance)
        rows = []
        for r in data:
            rows.append(tuple([r[c] for c in self.getColumnIds()]))
        return tuple(rows)
    
    def resetFixedRows(self, instance, data):
        """ See that fixed rows exists.
        
        Go through data (list of rows/dict) and add fixed rows if they are missing.
        
        1. Go through all fixed rows
        2. See if the key column of the fixed row has value in user data 
        3. If the row is missing, (re)append it
        
        @param data user set data
        @return modified data w/fixed rows present
        """
        
        # is fixed row property used
        if hasattr(self, "fixed_rows") and self.fixed_rows != None:
                                                                  
            if isinstance(self.fixed_rows, basestring):
                # fixed rows is a name of a member function
                
                try:
                    func = getattr(instance, self.fixed_rows)
                except AttributeError:
                    raise AttributeError, "Class %s is missing fixed row data function %s" % (str(instance), self.fixed_rows)
                                
                fixedRowsData = func()
                
            else:                                       
                # fixed rows is a direct value
                
                fixedRowsData = self.fixed_rows
                
            newRows = list(data[:])
                                            
            for fixedRow in fixedRowsData:                    
                # go through data set and see if the fixed key value exists
                keyValue = fixedRow.initialData[fixedRow.keyColumn]
                                                                        
                exist = False
                    
                for row in data:
                    if row.has_key(fixedRow.keyColumn):
                        if row[fixedRow.keyColumn] == keyValue:
                            # row exists and has user set value
                            exist = True
                            break
                            
                if not exist:
                    # initialize fixed data
                    newRows.append(fixedRow.initialData)
                                                              
            return tuple(newRows)
        else:
            # fixed rows behavior is disabled
            return data
    
        
class FixedRow:    
    """ Row which is always present at DataGridField data.
    
    This is a useful use case for situations where user must be
    forced to fill in some rows containing pre-set data. An example could 
    be the filling of programming language knowledge in CV. Languages are preset
    and user fills in his/her experience. User can also add some weird languages outside 
    pre-set languages.
    
    Instead of going with normal field.default behavior, fixed rows allow some flexibility
    when changing the fixed data set after item initialization. For example,
    the set of programming languages can be updated and user refills missing values
    to his/her CV.
    """
        
    def __init__(self, keyColumn, initialData):
        """
        @param initialData Dictionary for the row when user has deleted the fiexd row/item is initialized
        @param keyColumn Column which existence of value determines the need for a fixed row 
        """
        self.keyColumn = keyColumn
        self.initialData = initialData

registerField(DataGridField,
              title='DataGridField',
              description=('Used for storing tabular string data'))



"""
    Archetypes items used in DataGridField unit testing and examples
    
"""
# Zope imports
from AccessControl import ClassSecurityInfo

# Plone imports
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import *

# Local imports
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.RadioColumn import RadioColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.DataGridField import FixedRow
from Products.DataGridField.HelpColumn import HelpColumn

class DataGridDemoType(BaseContent):
    """ Very simple DataGridField demo.
    
    This class is used in unit testing, mainly to check old interface compatibility 
    (without widget column definitions). Please see the more complex examples below.
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((

        DataGridField('DemoField',
                searchable=True, # One unit tests checks whether text search works
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),                
                default=[
                {'column1':'a', 'column2':'b', 'The third':'c'},
                {'column1':'d', 'column2':'e', 'The third':'f'}
                ]
            ),
        ),                                 
    )
                                                                      
    meta_type = portal_type = archetype_name = 'DataGridDemoType'
    
registerType(DataGridDemoType)    

class DataGridDemoType2(BaseContent):
    """ Demo for different DataGridWidget columns
    
    This class is used in unit testing
    
    Check manual that:
        - Rows are inserted automatically when a value is filled in
        - Select column has sample 2 as a default value    
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((
                                  
        DataGridField('AutoInsertDemoField',
                searchable=True, # One unit tests checks whether text search works                      
                columns=("column1", "column2", "column3"),
                allow_empty_rows = False, # Must be false to make auto insert feature perform correctly
                widget = DataGridWidget(
                    auto_insert = True,     
                    description="Automatically insert new rows when the last row is being filled. When you edit the last row, a new row is created automatically.",
                    columns={
                    },
                 ),                 
         ),
                                  
        DataGridField('DemoField2',
                searchable=True, # One unit tests checks whether text search works
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    description="Set default values for created rows. Choose SelectColumn value from the default dictionary",
                    columns={
                        'column1' : Column("Toholampi city rox"),
                        'column2' : Column("My friendly name", default="Some default text"),
                        'select_sample' : SelectColumn("Friendly name", vocabulary="getSampleVocabulary", default="sample2")
                    },
                 ),                 
         ),                                  
         
        DataGridField('DemoField3',
                columns=("column1", "column2"),
                widget = DataGridWidget(
                    description="Test radio and checkbox columns",
                    columns={
                        'column1' : RadioColumn("Radio column", vocabulary="getSampleVocabulary"),
                        'column2' : CheckboxColumn("Checkbox column")                    
                    },
                 ),                 
         ),                        

        DataGridField('DemoField4',
                columns=("text_column", "help_column"),
                widget = DataGridWidget(
                    description="Help column test",
                    columns={
                        'text_column' : Column("Text column"),
                        # Help is help.pt
                        'help_column' : HelpColumn("Help", "See help here", "help", "info.gif")
                    },
                 ),                 
         ),                                  

                
        ))
    
    meta_type = portal_type = archetype_name = 'DataGridDemoType2'
    
    def getSampleVocabulary(self):
        """Get a sample vocabulary
        """
        return DisplayList(
    
            (("sample", "Sample value 1",),
            ("sample2", "Sample value 2",),))

registerType(DataGridDemoType2)

class InvalidDataGridDemoType(BaseContent):
    """ DataGridField declaration with errors
    
    Errors should be detected run-time, with helpful error messages.
    
    This class is missing column definition select_sample in DataGridWidget        
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((
        DataGridField('DemoField',
                searchable = True,
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    columns={
                        'column1' : Column("Toholampi city rox"),
                        'column2' : Column("My friendly name"),
                    },                 
          ),),
                
        ))
    
    meta_type = portal_type = archetype_name = 'InvalidDataGridDemoType'        
    
registerType(InvalidDataGridDemoType)
    
    
class FixedRowsDemoType(BaseContent):
    """ Demostrate fixed rows usage
    
    This class is used in unit testing
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((

        DataGridField('DemoField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                fixed_rows = [
                    FixedRow(keyColumn="column1", initialData = { "column1" : "must-exist-1", "column2" : "bbb" }),
                    FixedRow(keyColumn="column2", initialData = { "column1" : "ddd", "column2" : "must-exist-2" }),
                ]
            ),
                                  
        DataGridField('RestrictedField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                allow_insert=False,
                allow_delete=False,
                allow_reorder=False,
            ),                                  
                                  
        DataGridField('predefinedSkills',
            searchable=True,          
            columns=('skill', 'level'),
            fixed_rows = "getPredefinedSkillsData",
            allow_delete = False,
            allow_insert = False,
            allow_reorder = False,
            widget = DataGridWidget(
                label="Skills",
                description="Language/technology/tool/method for which employer has special interest",
                columns= {
                    "skill" : FixedColumn("Skill"),
                    "level" : RadioColumn("Level", vocabulary="getSkillLevels")
                }
            ),            
            ),                                  
        ))
        
    meta_type = portal_type = archetype_name = 'FixedRowsDemoType'
        
    def getSkillLevels(self):
        return DisplayList(    
        (("bad", "Bad",),
         ("good", "Good",),
        ))
        
    def getPredefinedSkillsData(self):      
        """ Generate fixed row key information """
        skills = [ "Python", "Perl", "XML", "Java", "Plone" ]
        rows = []
        for skill in skills:            
            rows.append(FixedRow(keyColumn="skill", initialData={"skill" : skill, "level" : "bad"}))
            
        return rows
    
    
registerType(FixedRowsDemoType)    
    






"""

    DataGridField initialzer.

    Copyright 2006-2007 DataGridField authors.
    
    Load all submodules and perform Zope security initialize for them.

"""

from __future__ import nested_scopes
__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'epytext'


# Plone imports
from Products.CMFCore.DirectoryView import registerDirectory
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.utils import ContentInit
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.atapi import process_types

# Local imports
from Products.DataGridField.DataGridWidget  import DataGridWidget
from Products.DataGridField.DataGridField  import DataGridField
from Products.DataGridField.DataGridField  import FixedRow
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.RadioColumn import RadioColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.LinkColumn import LinkColumn
from Products.DataGridField.HelpColumn import HelpColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn
from Products.DataGridField import validators

from Products.DataGridField.config import PKG_NAME, GLOBALS

registerDirectory('skins', GLOBALS)

def initialize(context):
    # Example content type initialization
    import Products.DataGridField.examples
    content_types, constructors, ftis = process_types(listTypes(PKG_NAME), PKG_NAME,)

    ContentInit(
        '%s Content' % PKG_NAME,
        content_types = content_types,
        permission = AddPortalContent,
        extra_constructors = constructors,
        fti = ftis,
        ).initialize(context)

try:
    from Products.CMFPlone.migrations import v3_1
except ImportError:
    HAS_PLONE31 = False
else:
    HAS_PLONE31 = True

try:
    from Products.CMFPlone.migrations import v3_0
except ImportError:
    HAS_PLONE30 = False
else:
    HAS_PLONE30 = True



"""
    Data grid UI component
            
    Copyright 2006-2007 DataGridField authors
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'epytext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from Products.DataGridField.Column import Column

class DataGridWidget(TypesWidget):
    """ Render a table with user addable rows

        Properties:
          - columns:
            - Dictionary of "column id" : Column instance mappings:
                - See *Column class documentation for initialization
                  parameters
          - helper_js:
              - Javascript used with the widget
          - helper_css:
              - CSS file used with the widget         
          - auto_insert
              - Automatically add new rows when the last row is being edited.
                This doesn't work if the 
                
    """

    _properties = TypesWidget._properties.copy()

    _properties.update({
        'macro' : "datagridwidget",
        'helper_css': ('datagridwidget.css',),
        'helper_js': ('datagridwidget.js',),
        'show_header' : True,
        'auto_insert': False,
        'columns' : {}, # Sequence of Column instances                           
        })


    security = ClassSecurityInfo()

    security.declarePublic('getColumnLabels')
    def getColumnLabels(self, field, context):
        """ Get user friendly names of all columns """

        columnDefinitions = getattr(self, 'columns', {})
        
        if len(columnDefinitions) == 0:
            # old way of getting column names
            columnNames = getattr(self, 'column_names', [])
            if columnNames:
                return columnNames
            else:
                return field.getColumnIds()


        names = []

        for id in field.getColumnIds():
            # Warn AT developer about his/her mistake
            if not id in columnDefinitions:
                raise AttributeError, "DataGridWidget missing column definition for " + id + " in field " + field.getName()

            col = self.columns[id]
            names.append(col.getLabel(self, context))

        return names

    security.declarePublic('getColumnDefinition')
    def getColumnDefinition(self, field, id):
        """ Return Column instance for column id """

        if id in getattr(self, 'columns', {}).keys():
            return self.columns[id]

        # Backwards compatability/shortcut
        if id in field.columns:
            label = id
            columnNames = getattr(self, 'column_names', None)
            if columnNames and len(columnNames) == len(field.columns):
                idx = list(field.columns).index(id)
                label = columnNames[idx]
                
            return Column(label)

        raise KeyError, "Tried to look up missing column definition for: " + str(id)

    security.declarePublic('getColumnDefs')
    def getColumnDefs(self, field, instance):
        """ Get all column definitions for a DataGridField.
        
            @param field Field definition 
            @param instance Context object who has the field
        
            @return formatted column definitions as dict of { id, label, visible }
        """
        result = []
        
        columns = getattr(self, 'columns', {})
        
        for id in field.columns:
            c = self.getColumnDefinition(field, id)
            if c is None:
                raise KeyError, "Tried to look up missing column definition for: " + str(id)
            item = {'id':id, 'label':'', 'visible':True}
            visible = getattr(c, 'visible', True)
            item['visible'] = visible
            item['label'] = c.getLabel(instance, self)
            result.append(item)
        
        return result

    def getUserFriendlySelectionItem(self, context, item, vocab):
        """Look up the given item in the vocab and return the value, translated
        if necessary. Return an empty string if item is empty or None.
        """
        if item == None or item == '':
            return ""
        return context.translate(vocab.getMsgId(item), default=vocab.getValue(item))
 
 
    security.declarePublic('isAutoInsertEnabled')
    def isAutoInsertEnabled(self):   
        return self.auto_insert

    security.declarePublic('isDeleteEnabled')
    def isDeleteEnabled(self, context, field):
        """ Can user delete rows from DGW 
        
        Called by template
        """
        return field.allow_delete
    
    security.declarePublic('isInsertEnabled')
    def isInsertEnabled(self, context, field):
        """ Can user insert new rows to DGW 
        
        Called by template
        """
        return field.allow_insert

    security.declarePublic('isReorderingEnabled')
    def isReorderEnabled(self, context, field):
        """ Can user reorder rows in DGW 
        
        Called by template
        """
        return field.allow_reorder
    
    security.declarePublic('hasAddButton')    
    def hasAddButton(self, context, field):
        """ Show explict Add row button in DGW """
        return self.show_add_button
    
    security.declarePublic('hasHeader')    
    def hasHeader(self, context, field):
        """ Render columns names in view mode"""
        return self.show_header
    
    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """ Manipulate form input data 
        
        Data coming from widget code goes through process_form  
        before stored to field.
        
        For example, radio button cells need to be propeply changed from
        checked values to one chosen id for saving.
        """
        
        value = TypesWidget.process_form(self, instance,
            field, form, empty_marker, emptyReturnsMarker)
                
        if value == None or len(value) == 0:
            return value
        
        newData = value[0]
        
        # Column code hook to form data and manipulate
        # it propeply where TypesWidget.process_form doesn't
        # have required functionality
        for columnId in getattr(self, 'columns', {}).keys():
            columnDefinition = self.getColumnDefinition(field, columnId)
            newData = columnDefinition.processCellData(form, newData, instance, field, columnId)            
        
        # Clean up the last empty row (automatically inseted)
        # if auto_insert is enabled
        if self.isAutoInsertEnabled() and len(newData) > 1:
            lastRow = newData[len(newData) - 1]
            hasContent = False
            for val in lastRow.values():
                if val != None and val != "":
                    hasContent = True
                        
            if not hasContent:
                newData = newData[:-1]
            
            
        return (newData, value[1])
    
    
    def savePostbackData(self, REQUEST, context, field, formData):
        """ Gets the value of a cell
        
        We cannot solely rely DataGridField.get which 
        does ObjectField.get(), since form postback data
        can contain custom fields which are not directly mapped
        like Zope expects (e.g. fields with more than one <input>)        
        
        XXX Need refactoring/clean up
        """        
        
       
        # Column code hook to form data and manipulate
        # it propeply where TypesWidget.process_form doesn't
        # have required functionality
        for columnId in getattr(self, 'columns', {}).keys():
            columnDefinition = self.getColumnDefinition(field, columnId)
            formData = columnDefinition.processCellData(REQUEST, formData, context, field, columnId)
                    

__all__ = ('DataGridWidget')

registerWidget(DataGridWidget,
               title='Data Grid',
               description=('A spreadsheet like table'),
               used_for=('Products.DataGridField.DataGridField',)
               )




from unittest import TestSuite, makeSuite
from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase


class TestValidation(ATSiteTestCase):

    def test_registration(self):
        from Products.validation import validation
        v = validation.validatorFor('isDataGridFilled')
        self.failUnlessEqual(v((0, 1)), 1)
        self.failUnlessEqual(v([0, 1]), 1)
        self.failIfEqual(v([1]), 1)
        self.failIfEqual(v(0), 1)


def test_suite():
    suite = TestSuite()
    
    suite.addTest(makeSuite(TestValidation))

    return suite



import unittest
from zope.testing import doctest
from zope.testing import doctestunit

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)


def test_suite():
    return unittest.TestSuite([
        doctestunit.DocFileSuite('validators.txt',
                                 package='Products.DataGridField.tests',
                                 optionflags=OPTIONFLAGS),
        ])



"""
            
    Copyright 2006 Red Innovation
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.DataGridField.Column import Column
from Products.DataGridField.utils import makeAbsoluteLink, makeRelativeLink

class LinkColumn(Column):
    """ Defines DataGridField column with link descriptions and targets
    
    Description and link is stored as one text entry, in of
    form which is used by many Wikis.
    
    Format:
        "description" "|" "link"     
        
    If links begin with http:// they are absolute. Otherwise
    links are relative to the portal root. 
    Managing absolute and relative links are done through utils module
    functions.
    """

    security = ClassSecurityInfo()
    
    
    def __init__(self, title, linkClass=""):
        """ Create a Links
    
        @param linkClass CSS class for <a> links in view mode
               
        """
        Column.__init__(self, title)
        self.linkClass = linkClass

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_link_cell"

    security.declarePublic('getLinkClass')
    def getLinkClass(self):
        return self.linkClass
    
    security.declarePublic('getLink')
    def getLink(self, instance, cell):
        """ Extract link address from cell data """
        
        if cell == None:
            return ""
        
        splitted = cell.split("|")
        if len(splitted) >= 2:
            return makeAbsoluteLink(splitted[1], instance)
        return ""
    
    security.declarePublic('getRelativeLink')
    def getRelativeLink(self, instance, cell):
        """ Extract link address from cell data """
        
        if cell == None:
            return ""
        
        splitted = cell.split("|")
        if len(splitted) >= 2:
            return makeRelativeLink(splitted[1], instance)
        return ""    
    
    security.declarePublic("getDescription")
    def getDescription(self, cell):
        """ Extract link description from cell data """
                
        if cell == None:
            return ""
        
        splitted = cell.split("|")
        if len(splitted) >= 1:
            return splitted[0]
        return ""
    
    
    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data
        
        Read form fields xxx_link and xx_desc and form one
        description from them.
        """
        
        # scan all rows and build desc|link string pair for
        # fields
        newValue = []
        for row in value:
            
            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]
                
            if newRow.has_key(columnId + "_desc"):
                desc = newRow[columnId + "_desc"]
            else:
                desc = ""
            
            if newRow.has_key(columnId + "_link"):
                link = newRow[columnId + "_link"]
            else:
                link = ""
                
            link = makeRelativeLink(link, context)
            
            if desc or link:
                newRow[columnId] = "%s|%s" % (desc,link)
            else:
                # don't add | character alone
                # causes extra new row to appear
                newRow[columnId] = ""
                                                       
            newValue.append(newRow)
            
        return newValue             
        
    

# Initializes class security
InitializeClass(LinkColumn)



from zope.interface import Interface as Z3Interface

class DemoProfile(Z3Interface):
    """Marker interface for our demo GS profile."""


# Zope imports
from Interface import Interface

# DataGrid interfaces
class IDataGridField(Interface):
    """A DataGrid can be used to manipulate two-dimensional data
    """

    def getColumns(instance):
        """Get a tuple of column names, in the order they are held in the grid.
        """

    def getRowCount(instance):
        """Get the number of rows in the grid.
        """

    def getRow(instance, rowNumber):
        """Get the given row by number. The first row is rowNumber 0. Returns
        a dict of the row.
        """
        pass

    def getColumn(instance, columnName):
        """Get the given column by name as a tuple of values.
        """
        pass

    def search(instance, key=None, **kwargs):
        """Search for rows. If key is given look for this value in the first
        column. If key is not given, at least one kwarg must be given,
        specifying column names and values. For example,
            search(surname='Jones', title='Mr')
        will return all rows with the surname and title matching Mr. Jones.
        Always returns a tuple of dicts.
        """
        pass

    def lookup(instance, key, column, lookupColumn=None):
        """Look for the given key in the column specified by lookupColumn.
        If no lookupColumn is given, look in the first column. For the first row
        found, return the value stored in the corresponding column as given by
        the 'column' parameter. Returns None if the key could not be found.

        (if you've used Excel, this is similar to VLOOKUP)
        """
        pass

    def getAsDisplayList(instance, keyCol=None, valueCol=None):
        """Get two columns of each row as a DisplayList - the key columns is
        keyCol, and the value column is valueCol. If these are not given,
        use the first two columns, respectively.
        """
        pass

    def getAsGrid(instance):
        """Return a tuple of tuples - the outer tuple has one element
        per row in the grid, the inner tuple has one element per column
        in that row.
        """
        pass



"""Validator for DataGridFields.

For the DataGridFields, making them required is not enough as there is
always a hidden entry.  So we check if there is least one normal entry
and one hidden entry, so more than 1 entry in total.
"""

from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator


class DataGridValidator:
    """Validate as True when having at least one DataGrid item.
    """

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        try:
            length = len(value) - 1
        except TypeError:
            return ("Validation failed(%s): cannot calculate length "
                    "of %s.""" % (self.name, value))
        except AttributeError:
            return ("Validation failed(%s): cannot calculate length "
                    "of %s.""" % (self.name, value))
        if length < 1:
            return ("Validation failed(%s): Need at least one entry."
                    % self.name)
        return True


isDataGridFilled = DataGridValidator(
    'isDataGridFilled', title='DataGrid has entries',
    description='The DataGridField must have at least one entry.')
validation.register(isDataGridFilled)



"""

    Basic columns for DataGridField.

    Copyright 2006-2007 DataGridField authors


"""

from __future__ import nested_scopes

__author__ = "Mikko Ohtamaa < mikko@redinnovation.com>"
__docformat__ = 'epytext'

# Zope imports
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass


class Column:
    """ Stardard text edit column """

    security = ClassSecurityInfo()

    def __init__(self, label, default=None, default_method=None, label_msgid=None):
        """ Create a column
        
            @param label User visible name
            @param default Default value for new rows
            @param default_value Default function to generate the default value for new rows
        """
        self.label = label
        self.default = default
        self.default_method = default_method
        
        if label_msgid is None:
            label_msgid = label
            self.label_msgid = label_msgid

    security.declarePublic('getLabel')
    def getLabel(self, context, widget):
        """ User friendly name for the columnt 
        
        @param context Context where translation happens
        @param widget The parent widget of this column
        """
        
        
        # TODO: translation support disabled
                
        #if HAS_PLONE25:
            # No translation support yet
        #    return zope.i18n.translate(
        #        _(self.label_msgid, 
        #          self.label), 
        #          context=context)
        #else:
        #    context.translate(
        #            msgid   = self.label_msgid,
        #            domain  = widget.i18n_domain,
        #            default = self.label)
        return self.label
  
    
    security.declarePublic('getDefault')
    def getDefault(self, context):
        """ Default value for new rows """
        if self.default_method:
            try:
                func = getattr(context, self.default_method)
            except AttributeError:
                raise AttributeError, "Class %s has no default_method %s" % (str(context), self.default_method)

            return func()
            
            
        return self.default

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_text_cell"
    
    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data
        
        Column processing in forms may need special preparations for data if
        widgets use other than <input value> for storing their
        values in fields.
        
        @param form Submitted form, contains HTML fields
        @param context Archetypes item instance for the submitted form
        @param field Assigned field for this widget
        @param columnId Column what we are operating
        
        @return new values which are constructed by processing data
        """
        return value

# Initializes class security
InitializeClass(Column)



"""
    DataGridField quick install script
            
    Copyright 2006 DataGridField authors
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Python imports
import StringIO
from cStringIO import StringIO
import string

from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes import listTypes

# Config
from Products.DataGridField.config import *
from Products.DataGridField import HAS_PLONE30

def install(self):
    out=StringIO()
    
    # get portal_setup tool
    setup_tool = getToolByName(self, 'portal_setup')
    
    # get the correct profile
    if HAS_PLONE30:
        setup_tool.runAllImportStepsFromProfile("profile-Products.DataGridField:default", purge_old=False)
    else:
        # BBB: We install our product by running a GS profile.  We use the old-style Install.py module 
        # so that our product works w/ the Quick Installer in Plone 2.5.x
        old_context = setup_tool.getImportContextID()
        setup_tool.setImportContext('profile-Products.DataGridField:default_25x')
        setup_tool.runAllImportSteps()
        setup_tool.setImportContext(old_context)
    
    # if INSTALL_DEMO_TYPES:
    #     installTypes(self, out, classes, PKG_NAME)
    
    out.write('Installation completed.\n')
    return out.getvalue()

def uninstall(self):
    out=StringIO()
    return out.getvalue()




"""

    Column with help for DataGridField 

    Written by Juan Grigera <juan@grigera.com.ar>.
    
"""

from __future__ import nested_scopes
__docformat__ = 'epytext'
__author__ = "Juan Grigera <juan@grigera.com.ar>"

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.CMFCore.permissions import View
from Products.DataGridField.Column import Column


class HelpColumn(Column):
    """ Help column support. 
    
    Behaves like normal text cell, but has a help pop-up icon next to it.
    """
    
    security = ClassSecurityInfo()
        
    def __init__(self, title, helper_text, script, icon):
        """ Create a HelpColumn
        
        """
        Column.__init__(self, title)
        self.helper_text = helper_text
        self.helper_url = script
        self.icon = icon
        
    
    security.declareProtected(View, 'getVocabulary')
    def getVocabulary(self, instance):
        """ Gets this column vocabulary for specific Archetypes instance
        """
        try:
            func = getattr(instance, self.vocabulary)
        except AttributeError:
            raise AttributeError, "Class %s is missing vocabulary function %s" % (str(instance), self.vocabulary)
                      
        return func()
        
    
    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_help_cell"
    
    security.declarePublic('getHelperUrl')
    def getHelperUrl(self):
        """ Return url to open"""
        return self.helper_url
        
    security.declarePublic('getHelperText')
    def getHelperText(self):
        """ Return help text"""
        return self.helper_text
    
    security.declarePublic('getIcon')
    def getIcon(self):
        return self.icon
        
# Initializes class security 
InitializeClass(HelpColumn)



"""

    Small utility functions for DGF
            
    Copyright 2006 Red Innovation
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

from AccessControl import ModuleSecurityInfo, allow_module
from Products.CMFCore.utils import getToolByName


module = "Products.DataGridField.utils"
security = ModuleSecurityInfo(module)
allow_module(module)

security.declarePublic("makeAbsoluteLink")
def makeAbsoluteLink(link, context):   
    """ Convert site relative links to absolute URLs
    
    Absolute links are passed through as is.
    Assume all http:// prefixed links are absolute.
    """
    
    if link == "":
        # this prevents makeAbsoluteLink not to 
        # do nasty things with empty fields
        return ""
    
    # see if the link is absolute 
    if link.startswith("http://"):
        return link
    else:
        portal_url = getToolByName(context, "portal_url")
        if link[:1]  == "/":            
            return portal_url() + link
        else:
            # if first character is / don't add it twice
            return portal_url() + "/" + link

security.declarePublic("makeRelativeLink")
def makeRelativeLink(link, context):   
    """ Convert absolute (copy-pasted) links to site relative links
    
    If user enters absolute link which points to current portal,
    convert link to relative link so it survives portal relocation.    
    
    Assume are http:// prefixed links are absolute.    
    """

    # check for absolute, avoid extra work below
    if link.startswith("http://"):            
        portal_url = getToolByName(context, "portal_url")()
        if link.startswith(portal_url):
            return link[len(portal_url):]
        
    # if first character is / don't add it twice
    return link
                



"""

    Radio button column 

    Copyright 2006 Red Innovation
    
    Licensed under GPL.

"""

from __future__ import nested_scopes
__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'reStructuredText'

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.DataGridField.SelectColumn import SelectColumn

class RadioColumn(SelectColumn):
    """ Allow user select one from many options using radio buttons.
    
    WARNING: Does not work with validation. All field values will be cleared
    when validation fails on the edit page submit. This is a limitation in
    the current architecture (Archetypes / ZPublisher / DataGridField).
    
    Some explanation hints:
    
    In::
        <input type="radio" name=""
    name must be unique among radio button group. 
    
    This is problematic. ZPublisher cannot associate radio buttons to :records 
    groups since :records groups are formed as::
        
        name string:${fieldName}.${column}:records;
    
    ZPublisher's :records naming convention doesn't allow different per row
    radio button groups. Instead, we assign an unique name for reach
    radio button cell using::
        
        name string:${fieldName}.${column}.${repeat/rows/number};
        
    Column.processCellData parses form data and propeply combines field value 
    from radio buttons so that  Archetypes' field framework correctly understand
    the set value.
           
    """
    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_radio_cell"

    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data
        
        Read special table for radio button columns from form data.
        The selected radio button cell id is placed as a cell value.
        """
        
        newValue = []
        
        #print "form value:" + str(form)
        
        for row in value:
            
            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]
                        
            orderIndex = row["orderindex_"]                        
            cellId = "%s.%s.%s" % (field.getName(), columnId, orderIndex)
            if form.has_key(cellId):
                # If radio button is set in HTML form
                # it's id appears in form of field.column.orderIndex
                newRow[columnId] = form[cellId]
                
            newValue.append(newRow)
            
        return newValue

        
        
        
# Initializes class security
InitializeClass(RadioColumn)



from setuptools import setup, find_packages
import os

version = '1.6rc1'

setup(name='Products.DataGridField',
      version=version,
      description="A table input component for Plone.",
      long_description=open(os.path.join("Products", "DataGridField", "README.txt")).read() + "\n" +
                       open(os.path.join("Products", "DataGridField", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Plone",
        "Framework :: Zope2",
        ],
      keywords='Plone DataGridField Archetypes',
      author='Jarn',
      author_email='info@jarn.com',
      maintainer='Mikko Ohtamaa',
      maintainer_email='info@redinnovation.com',
      url='http://plone.org/products/datagridfield',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



"""

    Fixed text column 

    Copyright 2006 Red Innovation
    
    Licensed under GPL.

"""

from __future__ import nested_scopes
__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'reStructuredText'

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.DataGridField.Column import Column

class FixedColumn(Column):
    """ Column with non-changeable text
    
    Useful with DataGridField.fixed_row property in some use cases.
    """
    security = ClassSecurityInfo()

    def __init__(self, label, default=None, label_msgid=None, visible=True):
        """ Create a column
        
            @param hide Hide column from displaying
        """
        Column.__init__(self, label, default, label_msgid)
        self.visible = visible

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_fixed_cell"

                
# Initializes class security
InitializeClass(FixedColumn)



"""

    Checkbox column 

    Copyright 2006, 2007 Radim Novotny
    
    Licensed under GPL.

"""

from __future__ import nested_scopes
__author__ = "Radim Novotny"
__docformat__ = 'epytext'

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.DataGridField.Column import Column

class CheckboxColumn(Column):
    """ Allow user select one from on/off options using check buttons.

    Implementation based on RadioColumn.
    
    WARNING - Please note, the current implementation does not work when 
    some field on the form raises validation error. In this case all checkboxes 
    are cleared. Do not use CheckboxColumn if your form has some required fields
    or validators active.
    
    There are no on/off values. Widget view displays HTML character with code
    &#10004; (checkmark) or - 
    """
    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_checkbox_cell"

    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data
        """
        
        newValue = []
        
        for row in value:
            
            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]
                        
            orderIndex = row["orderindex_"]                        
            cellId = "%s.%s.%s" % (field.getName(), columnId, orderIndex)
            if form.has_key(cellId):
                # If check button is set in HTML form
                # it's id appears in form of field.column.orderIndex
                # field value is hardcoded to '1'
                newRow[columnId] = form[cellId]
            else:
                # if item is not in form, user did not check it.
                newRow[columnId] = ''
                
            newValue.append(newRow)
            
        return newValue

        
        
        
# Initializes class security
InitializeClass(CheckboxColumn)



"""

    Dropdown column definition for DataGridField

    Copyright 2006 Mikko Ohtamaa


"""

from __future__ import nested_scopes
__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'epytext'


from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.DataGridField.Column import Column


class SelectColumn(Column):
    """ Defines column with dropdown menu cells in DataGridField """

    security = ClassSecurityInfo()

    def __init__(self, title, vocabulary, default=None):
        """ Create a SelectColumn

        @param vocabulary Vocabulary method name. This method is called
               from Archetypes instance to get values for dropdown list.
        """
        Column.__init__(self, title, default=default)
        self.vocabulary = vocabulary


    security.declarePublic('getVocabulary')
    def getVocabulary(self, instance):
        """ Gets this column vocabulary for specific Archetypes instance
        """
        try:
            func = getattr(instance, self.vocabulary)
        except AttributeError:
            raise AttributeError, "Class %s is missing vocabulary function %s" % (str(instance), self.vocabulary)

        return func()


    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_select_cell"


# Initializes class security
InitializeClass(SelectColumn)



## Python Script "queryDataGrid"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=fieldName, method, args
##title=Query a data grid
##

from Products.CMFPlone.utils import safe_hasattr

# Find the field on this item or any parent
# Apply the given method (passed in as a string) with the given arguments
# (as a list) and return the results.


obj = context
while obj.aq_parent is not None:
    if safe_hasattr(obj, 'Schema') and obj.Schema().has_key(fieldName):
        break
    else:
        obj = obj.aq_parent

query = getattr(obj.getWrappedField(fieldName), method)
return query(obj, *args)


"""
    
    Unit testing for DataGridField product
    
    Copyright 2006 DataGridField authors
    
"""
    
import os, sys
import logging

# Output debug log
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
logging.getLogger("DataGridField").setLevel(logging.DEBUG)

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.DataGridField.tests.DataGridTestCase import DataGridTestCase
from Products.Archetypes.public import *
from Products.CMFCore.utils import getToolByName

from Products.DataGridField.utils import makeAbsoluteLink, makeRelativeLink

class TestInstallation(DataGridTestCase):

    def afterSetUp(self):
        types = self.portal.portal_types.objectIds()        
        self.failUnless('DataGridDemoType' in types, "Set INSTALL_DEMO_TYPES=True in config.py")        
        pass

    def testPortalTypes(self):
        types = self.portal.portal_types.objectIds()
        self.failUnless('DataGridDemoType' in types)

    def testPortalSkins(self):
        skins = self.portal.portal_skins.objectIds()
        self.failUnless('DataGridWidget' in skins)

    def testAddingDemo(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        self.failUnless('foo' in self.folder.objectIds())

    def testSettingField(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        vals = [{'column1':'joe'}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(len(self.folder.foo.getDemoField()),1)
        
        row1 = self.folder.foo.getDemoField()[0]        
        self.failUnless(row1.has_key("column1"), "Row was " + str(row1))
        pair1 = row1['column1']
        self.assertEqual(pair1,vals[0]['column1'])

    def testSettingFieldWithOrder(self):
        self.folder.invokeFactory('DataGridDemoType','foo')
        vals = [{'column1':'joe', 'orderindex_' : 1}, {'column1':'jack', 'orderindex_' : 0}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(len(self.folder.foo.getDemoField()),2)
        self.assertEqual(self.folder.foo.getDemoField()[0]['column1'], vals[1]['column1'])
        self.assertEqual(self.folder.foo.getDemoField()[1]['column1'], vals[0]['column1'])

    def testSettingEmptyRows(self):
        """ It should be possible to set empty rows """
        self.folder.invokeFactory('DataGridDemoType','foo')
        
        vals = ({'The third':'','column1':'xxx','column2':'',},)
        self.folder.foo.setDemoField(vals)        
        self.assertEqual(self.folder.foo.getDemoField(),vals)        
        
        vals = ({'The third':'','column1':'','column2':'',},)
        self.folder.foo.setDemoField(vals)
        self.assertEqual(self.folder.foo.getDemoField(),vals)

    def testRenderingDoesNotFail(self):
        """ See if a page containing DGF will output HTML without exceptions in view mode """  
        self.folder.invokeFactory('DataGridDemoType','foo')
        self.failUnless(self.folder.foo())

class TestGridAPI(DataGridTestCase):

    def afterSetUp(self):

        self.folder.invokeFactory('DataGridDemoType', 'demo')
        self.demo = self.folder.demo
        self.demo.setDemoField([{'column1':'a', 'column2':'b', 'The third':'c'},
                                {'column1':'d', 'column2':'e', 'The third':'f'}])
        self.field = self.demo.getField('DemoField')

    def testGetColumns(self):
        columns = self.field.getColumns(self.demo)
        self.assertEqual(columns[0], 'column1')
        self.assertEqual(columns[1], 'column2')
        self.assertEqual(columns[2], 'The third')

    def testGetRowCount(self):
        self.assertEqual(self.field.getRowCount(self.demo), 2)

    def testGetRow(self):
        self.assertEqual(self.field.getRow(self.demo, 1), {'column1':'d', 'column2':'e', 'The third':'f'})

    def testGetColumn(self):
        self.assertEqual(tuple(self.field.getColumn(self.demo, 'column2')), ('b', 'e',))

    def testSearchKey(self):
        self.assertEqual(self.field.search(self.demo, key='d'), ({'column1':'d', 'column2':'e', 'The third':'f'},))

    def testSearchColumn(self):
        self.assertEqual(self.field.search(self.demo, column2='e'), ({'column1':'d', 'column2':'e', 'The third':'f'},))

    def testSearchWithNoResults(self):
        self.assertEqual(self.field.search(self.demo, 'foo'), ())

    def testLookup(self):
        self.assertEqual(self.field.lookup(self.demo, 'd', 'column2'), 'e')

    def testLookupWithKeyColumn(self):
        self.assertEqual(self.field.lookup(self.demo, 'b', 'column1', 'column2'), 'a')

    def testLookupNotFound(self):
        self.assertEqual(self.field.lookup(self.demo, 'x', 'The third'), None)

    def testGetAsDisplayList(self):
        lst = DisplayList()
        lst.add('a', 'b')
        lst.add('d', 'e')
        self.assertEqual(self.field.getAsDisplayList(self.demo), lst)

    def testGetAsDisplayListWithCustomColumns(self):
        lst = DisplayList()
        lst.add('c', 'a')
        lst.add('f', 'd')
        self.assertEqual(self.field.getAsDisplayList(self.demo, 'The third', 'column1'), lst)

    def testGetAsGrid(self):
        self.assertEqual(tuple(self.field.getAsGrid(self.demo)), (('a', 'b', 'c',),
                                                                  ('d', 'e', 'f'),))

    def testQueryDataGrid(self):
        self.assertEqual(self.demo.queryDataGrid('DemoField', 'lookup', ['d', 'column2']), 'e')

    def testSearch(self):
        catalog_tool = self.portal.portal_catalog
        self.folder.invokeFactory('DataGridDemoType2', 'searchme')
        self.folder.searchme.setDemoField2([{'column1':'randomSearchableWord', 'column2':'b', 'select_sample':'sample'},
                                           {'column1':'d', 'column2':'e', 'select_sample':'sample'}])
        self.folder.searchme.reindexObject()
        self.failUnless('randomSearchableWord' in self.folder.searchme.SearchableText())
        brains = catalog_tool(SearchableText = 'randomSearchableWord')                                   
        self.failUnless(len(brains) > 0)
        
        
class TestSpecialRowsBehavior(DataGridTestCase):    
    """ Test fixed rows property, deletion locking, etc. """

    def afterSetUp(self):
        self.folder.invokeFactory('FixedRowsDemoType', 'demo')
        self.demo = self.folder.demo        

        self.field = self.demo.getField('DemoField')        
        self.failUnless(self.field != None)
        
        self.restrictedField = self.demo.getField('predefinedSkills')
        self.failUnless(self.restrictedField != None, "No restricted field in demo type")
        
    def checkForCellValue(self, data, columnId, value):
        for row in data:
            if row.has_key(columnId):
                if row[columnId] == value:
                    return True
        self.fail("Missing value %s for column %s" % (value, columnId))

    def testFixedRows(self):    
        """ Test that fixed rows are there always and non-key values can be changed
        """
        
        # initial data get should be empty + fixed rows present    
        data = self.field.get(self.demo)
                
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")
    
    def testAddNonFixedRows(self):
        """ set more row and see that fixed rows are still present """
        self.demo.setDemoField([{'column1':'a', 'column2':'b', 'The third':'c'},
                                {'column1':'d', 'column2':'e', 'The third':'f'}])
        
        data = self.field.get(self.demo)                                
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")
        self.checkForCellValue(data, "column1", "a")        
        self.checkForCellValue(data, "column2", "e")                


    def testSetFixedRowValue(self):
        """ set non-key value of fixed row"""
        self.demo.setDemoField([{'column1':'must-exist-1', 'column2':'testvalue'}])
        data = self.field.get(self.demo)        
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")        
        self.checkForCellValue(data, "column2", "testvalue")
        
        # test setting data outside fixed row definition
        self.demo.setDemoField([{'column1':'naah', 'column2':'nooh'}])
        data = self.field.get(self.demo)                
        self.checkForCellValue(data, "column1", "naah")
        self.checkForCellValue(data, "column2", "nooh")
                
    def testDeleteRestrictedField(self):
        self.failUnless(self.restrictedField.widget.isDeleteEnabled(self.demo, self.restrictedField) == False)

    def testInsertRestrictedField(self):
        self.failUnless(self.restrictedField.widget.isInsertEnabled(self.demo, self.restrictedField) == False)

    def testReorderRestrictedField(self):
        self.failUnless(self.restrictedField.widget.isReorderEnabled(self.demo, self.restrictedField) == False)
 
class TestLinkUtils(DataGridTestCase):
    """ Test LinkColumn link transforms """
    
    def afterSetUp(self):
        self.folder.invokeFactory("DataGridDemoType", id="sample")

            
    def testMakeAbsoluteURL(self):
        """ Test converting site relative links to full URLs
        """
        
        context = self.folder.sample

        self.failUnless(makeAbsoluteLink("test", context) == "http://nohost/plone/test")
        self.failUnless(makeAbsoluteLink("/test", context) == "http://nohost/plone/test")
        self.failUnless(makeAbsoluteLink("http://test", context) == "http://test")
        
    def testMakeRelativeURL(self):
        """ Test converting full URLs to site relative links
        """

        context = self.folder.sample       
            
        self.failUnless(makeRelativeLink("http://nohost/plone/test", context) == "/test")        
        self.failUnless(makeRelativeLink("/test", context) == "/test")
        self.failUnless(makeRelativeLink("test", context) == "test")
        self.failUnless(makeRelativeLink("http://test", context) == "http://test")        
        
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestGridAPI))
    suite.addTest(makeSuite(TestSpecialRowsBehavior))
    suite.addTest(makeSuite(TestLinkUtils))    
    return suite

if __name__ == '__main__':
    framework()



from Testing import ZopeTestCase
from Products.DataGridField import HAS_PLONE31

ZopeTestCase.installProduct('DataGridField')

from Products.PloneTestCase import PloneTestCase

if HAS_PLONE31:
    # passing multiple extension profiles works for Plone 3.1.x
    PloneTestCase.setupPloneSite(extension_profiles=['Products.DataGridField:default','Products.DataGridField:example',])
else:
    PloneTestCase.setupPloneSite(products=["DataGridField",], extension_profiles=['Products.DataGridField:example',])

class DataGridTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()
